// (c) Copyright Microsoft Corporation.
// This source is subject to the Microsoft Permissive License.
// See http://www.microsoft.com/resources/sharedsource/licensingbasics/sharedsourcelicenses.mspx.
// All other rights reserved.

Type.registerNamespace('Dflying.Ajax');

Dflying.Ajax.PopupNotificationBehavior = function(element) {

    Dflying.Ajax.PopupNotificationBehavior.initializeBase(this, [element]);

    // property member values
    this._resizeEffectDuration = 0.3;
    this._showNotificationDuration = 3;
    this._resultLabelID = "";
    this._servicePath = "";
    this._serviceMethod = "";
    this._queryServiceInterval = 60000;
    
    // animiations
    this._resizeAnimateShow = null;
    this._moveAnimateShow = null;
    this._resizeAnimateHide = null;
    this._moveAnimateHide = null;
    
    // show notification timer and handler
    this._showNotificationTimer = null;
    this._showNotificationHandler = null;
    
    // query service timer and handler
    this._queryServiceTimer = null;
    this._queryServiceHandler = null;
    
    // mouse over & out handlers
    this._mouseOverHandler = null;
    this._mouseOutHandler = null;
    
    // result label
    this._resultLabel = null;
    
    // original height of the binding element
    this._originalHeight = 0;
    
    // visibility state of the this notification pane
    this._isVisiable = false;

}

Dflying.Ajax.PopupNotificationBehavior.prototype = {

    initialize : function() {
        // get binding element
        var element = this.get_element();
        
        // save original height of the binding element
        this._originalHeight = element.offsetHeight;
        
        // set binding element to invisiable
        element.style.height = 0;
        
        // call base constructor
        Dflying.Ajax.PopupNotificationBehavior.callBaseMethod(this, 'initialize');
        
        // init animations
        this._resizeAnimateShow = new AjaxControlToolkit.Animation.ResizeAnimation(
            element, this._resizeEffectDuration, 25, null, this._originalHeight, 'px');
        this._moveAnimateShow = new AjaxControlToolkit.Animation.MoveAnimation(
            element, this._resizeEffectDuration, 25, null, -this._originalHeight, true, 'px');
        this._resizeAnimateHide = new AjaxControlToolkit.Animation.ResizeAnimation(
            element, this._resizeEffectDuration, 25, null, 0, 'px');
        this._moveAnimateHide = new AjaxControlToolkit.Animation.MoveAnimation(
            element, this._resizeEffectDuration, 25, null, this._originalHeight, true, 'px');       
        
        // mouse over handler
        this._mouseOverHandler = Function.createDelegate(this, this._onMouseOver);
        $addHandler(element, 'mouseover', this._mouseOverHandler);
        
        // mouse out handler
        this._mouseOutHandler = Function.createDelegate(this, this._onMouseOut);
        $addHandler(element, 'mouseout', this._mouseOutHandler);
        
        // show notification timer setup
        this._showNotificationHandler = Function.createDelegate(this, this._onShowNotificationTimerTick);
        this._showNotificationTimer = new Sys.Timer();
        this._showNotificationTimer.add_tick(this._showNotificationHandler);
        this._showNotificationTimer.set_interval((this._showNotificationDuration + this._resizeEffectDuration) * 1000);
        
        // query service timer setup
        this._queryServiceHandler = Function.createDelegate(this, this._onQueryServiceTimerTick);
        this._queryServiceTimer = new Sys.Timer();
        this._queryServiceTimer.add_tick(this._queryServiceHandler);
        this._queryServiceTimer.set_interval(this._queryServiceInterval);
        this._queryServiceTimer.set_enabled(true);        
    },

    dispose : function() {
        // get binding element
        var element = this.get_element();
        
        if (this._mouseOverHandler) {
            $removeHandler(element, 'mouseover', this._mouseOverHandler);
            this._mouseOverHandler = null;
        }
        if (this._mouseOutHandler) {
            $removeHandler(element, 'mouseout', this._mouseOutHandler);
            this._mouseOutHandler = null;
        }
        
        if (this._resizeAnimateShow) {
            this._resizeAnimateShow.dispose();
            this._resizeAnimateShow = null;
        }
        if (this._moveAnimateShow) {
            this._moveAnimateShow.dispose();
            this._moveAnimateShow = null;
        }
        if (this._resizeAnimateHide) {
            this._resizeAnimateHide.dispose();
            this._resizeAnimateHide = null;
        }
        if (this._moveAnimateHide) {
            this._moveAnimateHide.dispose();
            this._moveAnimateHide = null;
        }
        
        if (this._showNotificationTimer) {
            this._showNotificationTimer.dispose();
            this._showNotificationTimer = null;
        }
        this._showNotificationHandler = null;
        
        if (this._queryServiceTimer) {
            this._queryServiceTimer.dispose();
            this._queryServiceTimer = null;
        }
        this._queryServiceHandler = null;

        Dflying.Ajax.PopupNotificationBehavior.callBaseMethod(this, 'dispose');
    },
    
    _onQueryServiceTimerTick : function(eventObject) {
        if (this._servicePath && this._serviceMethod) {
            // Call the helper web service
            Sys.Net.WebServiceProxy.invoke(
                this.get_ServicePath(), 
                this.get_ServiceMethod(), 
                false,
                null,
                Function.createDelegate(this, this._onQueryServiceMethodComplete)
            );
        }
    },
    
    _onQueryServiceMethodComplete : function(sender, eventArgs) {
        // get server side result
        var result = sender;
        
        if (!this._resultLabel)
        {
            this._resultLabel = $get(this._resultLabelID);
        }
        
        // update message
        if (result != null && result != "")
        {
            this._resultLabel.innerHTML = result;
        }
        
        // play show animations
        if (!this._isVisiable)
        {
            this._resizeAnimateShow.play();
            if (this.get_VerticalSide() == AjaxControlToolkit.VerticalSide.Bottom)
            {
                this._moveAnimateShow.play();
            }
            
            // start timer
            this._showNotificationTimer.set_enabled(true);
            this._isVisiable = true;
        }
    },
    
    _onShowNotificationTimerTick : function() {
        // disable timer
        this._showNotificationTimer.set_enabled(false);
        
        // play hide animations
        if (this._isVisiable)
        {
            this._resizeAnimateHide.play();
            if (this.get_VerticalSide() == AjaxControlToolkit.VerticalSide.Bottom)
            {
                this._moveAnimateHide.play();
            }
            this._isVisiable = false;
        }
    },
    
    _onMouseOver : function () {
        // stop animations in case of they are playing
        this._resizeAnimateShow.stop();
        this._resizeAnimateHide.stop();
        if (this.get_VerticalSide() == AjaxControlToolkit.VerticalSide.Bottom) {
            this._moveAnimateHide.stop();
            this._moveAnimateShow.stop();
        }
        
        // show notification pane
        this.get_element().style.height = this._originalHeight + 'px';
        
        // disable timer
        this._showNotificationTimer.set_enabled(false);
    },
    
    _onMouseOut : function () {
        // start timer again
        this._showNotificationTimer.set_enabled(true);
        this._isVisiable = true;
    },
    
    get_ResizeEffectDuration : function() {
        return this._resizeEffectDuration;
    },

    set_ResizeEffectDuration : function(value) {
        if (this._resizeEffectDuration != value) {
            this._resizeEffectDuration = value;
        }
    },
    
    get_ShowNotificationDuration : function() {
        return this._showNotificationDuration;
    },

    set_ShowNotificationDuration : function(value) {
        if (this._showNotificationDuration != value) {
            this._showNotificationDuration = value;
            if (this._showNotificationTimer) {
                this._showNotificationTimer.set_interval((this._showNotificationDuration + this._resizeEffectDuration) * 1000);
            }
        }
    },
    
    get_ResultLabelID : function() {
        return this._resultLabelID;
    },

    set_ResultLabelID : function(value) {
        if (this._resultLabelID != value) {
            this._resultLabelID = value;
            this._resultLabel = $get(this._resultLabelID);
        }
    },
    
    get_ServicePath : function() {
        return this._servicePath;
    },

    set_ServicePath : function(value) {
        if (this._servicePath != value) {
            this._servicePath = value;
        }
    },
    
    get_ServiceMethod : function() {
        return this._serviceMethod;
    },

    set_ServiceMethod : function(value) {
        if (this._serviceMethod != value) {
            this._serviceMethod = value;
        }
    },
    
    get_QueryServiceInterval : function() {
        return this._queryServiceInterval;
    },

    set_QueryServiceInterval : function(value) {
        if (this._queryServiceInterval != value) {
            this._queryServiceInterval = value;
            if (this._queryServiceTimer) {
                this._queryServiceTimer.set_interval(this._queryServiceInterval);
            }
        }
    }
}

Dflying.Ajax.PopupNotificationBehavior.registerClass('Dflying.Ajax.PopupNotificationBehavior', AjaxControlToolkit.AlwaysVisibleControlBehavior);
